/**
 * jjj-promise v1.1.0
 * author: JJJYY<jessey9527@gmail.com>
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.Promise = factory());
}(this, (function () { 'use strict';

  function _typeof(obj) {
    if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
      _typeof = function (obj) {
        return typeof obj;
      };
    } else {
      _typeof = function (obj) {
        return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
      };
    }

    return _typeof(obj);
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  //      
  var CONFIG = {
    PENDING: 0,
    FULFILLED: 1,
    REJECTED: 2
  };
  var noop = function noop() {};
  var identify = function identify(_) {
    return _;
  };
  function isFunc(x) {
    return typeof x === 'function';
  }
  function isObjectOrFunc(x) {
    return x !== null && 'object,function'.includes(_typeof(x));
  }
  function isThenable(x) {
    return isObjectOrFunc(x) && isFunc(x.then);
  }

  //      
  var browserGlobal = typeof window !== 'undefined' ? window : {};
  var BrowserMutationObserver = browserGlobal.MutationObserver || browserGlobal.WebKitMutationObserver;
  var isNode = typeof self === 'undefined' && typeof process !== 'undefined' && {}.toString.call(process) === '[object process]'; // test for web worker but not in IE10

  var isWorker = typeof Uint8ClampedArray !== 'undefined' && typeof importScripts !== 'undefined' && typeof MessageChannel !== 'undefined';
  var triggerTick;

  if (isNode) {
    triggerTick = process.nextTick;
  } else if (BrowserMutationObserver) {
    var iterations = 0;
    var callback = noop;
    var node = document.createTextNode('');
    var observer = new BrowserMutationObserver(function () {
      return callback();
    });
    observer.observer(node, {
      characterData: true
    });

    triggerTick = function triggerTick(cb) {
      callback = cb;
      node.data = ++iterations + '';
    };
  } else if (isWorker) {
    var channel = new MessageChannel();

    triggerTick = function triggerTick(cb) {
      channel.port1.onmessage = cb;
      channel.port2.postMessage(0);
    };
  } else {
    triggerTick = setTimeout;
  }

  var triggerTick$1 = triggerTick;

  //      
  var fulfilledDefaultFun = identify;

  var rejectedDefaultFun = function rejectedDefaultFun(reason) {
    throw reason;
  };

  function getThen(promise, x) {
    try {
      return x.then;
    } catch (e) {
      reject(promise, e);
    }
  }

  function addNextTick(promise, result) {
    triggerTick$1(function () {
      var i = 0;
      var sequence = promise._sequence;

      while (sequence[i]) {
        try {
          // $FlowFixMe
          resolve(sequence[i], (0, sequence[i + promise._status])(result));
        } catch (e) {
          reject(sequence[i], e);
        }

        i += 3;
      }

      promise._sequence = [];
    });
  }

  function fulfill(promise, value) {
    if (promise._status === CONFIG.PENDING) {
      addNextTick(promise, value);
      promise._result = value;
      promise._status = CONFIG.FULFILLED;
    }
  }

  function nextTick(promise) {
    if (promise._status !== CONFIG.PENDING) addNextTick(promise, promise._result);
  }

  function resolve(promise, x) {
    if (promise === x) {
      // x 与 promise 相等
      reject(promise, new TypeError("\u53C2\u6570 [".concat(x, "] \u4E0D\u80FD\u4E0E promise \u76F8\u7B49")));
    } else if (isObjectOrFunc(x)) {
      var _then = getThen(promise, x); // 存储then，以保证 then 单次访问


      if (isFunc(_then)) {
        var _alive = true;

        try {
          // $FlowFixMe
          _then.call(x, function resolvePromise(y) {
            _alive && resolve(promise, y);
            _alive = false;
          }, function rejectPromise(r) {
            _alive && reject(promise, r);
            _alive = false;
          });
        } catch (e) {
          _alive && reject(promise, e); // then 执行过程出错,若未执行 resolve/reject 则以 reject 处理该 promise
        }
      } else {
        fulfill(promise, x);
      }
    } else {
      fulfill(promise, x);
    }
  }
  function reject(promise, reason) {
    if (promise._status === CONFIG.PENDING) {
      addNextTick(promise, reason);
      promise._result = reason;
      promise._status = CONFIG.REJECTED;
    }
  } // 这才是 then 开始

  function then(promise) {
    var onFulfilled = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : fulfilledDefaultFun;
    var onRejected = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : rejectedDefaultFun;
    var sequence = promise._sequence;
    var length = sequence.length;
    sequence[length] = new promise.constructor(noop);
    sequence[length + CONFIG.FULFILLED] = isFunc(onFulfilled) ? onFulfilled : fulfilledDefaultFun;
    sequence[length + CONFIG.REJECTED] = isFunc(onRejected) ? onRejected : rejectedDefaultFun;
    nextTick(promise); // 对于已解决的 promise 需要将结果，在下次 tick 中返回给then

    return sequence[length];
  }

  //      
  function runPromise(promise, handle) {
    try {
      handle(function _resolve(value) {
        resolve(promise, value);
      }, function _reject(reason) {
        reject(promise, reason);
      });
    } catch (e) {
      reject(promise, e);
    }
  }

  var Promise$1 =
  /*#__PURE__*/
  function () {
    function Promise(handle) {
      _classCallCheck(this, Promise);

      _defineProperty(this, "_sequence", []);

      this._status = CONFIG.PENDING;
      runPromise(this, handle);
    }

    _createClass(Promise, [{
      key: "then",
      value: function then$$1(onFulfilled, onRejected) {
        return then(this, onFulfilled, onRejected);
      }
    }, {
      key: "catch",
      value: function _catch(onRejected) {
        return then(this, identify, onRejected);
      }
    }, {
      key: "finally",
      value: function _finally(onFinally) {
        return then(this, onFinally, onFinally);
      }
    }], [{
      key: "resolve",
      value: function resolve$$1(data) {
        return new Promise(function (resolve$$1) {
          return resolve$$1(data);
        });
      }
    }, {
      key: "reject",
      value: function reject$$1(reason) {
        return new Promise(function (resolve$$1, reject$$1) {
          return reject$$1(reason);
        });
      }
    }, {
      key: "all",
      value: function all(promises) {
        return new Promise(function (resolve$$1, reject$$1) {
          var result = [];
          var num = 0;

          function onFulfilled(value, i) {
            result[i] = value;
            num++; // $FlowFixMe

            if (num === promises.length) resolve$$1(result);
          } // $FlowFixMe


          promises.forEach(function (p, i) {
            return isThenable(p) ? p.then(function (value) {
              return onFulfilled(value, i);
            }, reject$$1) : onFulfilled(p, i);
          });
        });
      }
    }, {
      key: "race",
      value: function race(promises) {
        return new Promise(function (resolve$$1, reject$$1) {
          return (// $FlowFixMe
            promises.forEach(function (p) {
              return isThenable(p) ? p.then(resolve$$1, reject$$1) : resolve$$1(p);
            })
          );
        });
      }
    }]);

    return Promise;
  }();

  return Promise$1;

})));
